@@ -19,10 +19,10 @@
  */
 
 #include "application.h"
-#include "dialog_new_database.h"
-#include "dialog_progress_creating.h"
-#include "translation/dialog_change_language.h"
-#include "translation/window_translations.h"
+#include <glom/libglom/dialog_progress_creating.h>
+#include <glom/translation/dialog_change_language.h>
+#include <glom/translation/window_translations.h>
+#include <glom/utility_widgets/filechooserdialog.h>
 #include <glom/libglom/utils.h>
 #include <libgnome/gnome-help.h> //For gnome_help_display
 #include "config.h" //For VERSION.
@@ -47,6 +47,7 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml
   m_window_translations(0),
   m_menu_tables_ui_merge_id(0),
   m_menu_reports_ui_merge_id(0),
+  m_ui_save_extra_newdb_selfhosted(false),
   m_show_sql_debug(false)
 {
   try
@@ -324,6 +325,11 @@ void App_Glom::init_menus()
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_reports));
 
+  action = Gtk::Action::create("GlomAction_Menu_Developer_Script_Library", _("Script _Library"));
+  m_listDeveloperActions.push_back(action);
+  m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_script_library));
+
+
   action = Gtk::Action::create("GlomAction_Menu_Developer_Layout", _("_Layout"));
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_layout));
@@ -369,6 +375,7 @@ void App_Glom::init_menus()
     "        <menuitem action='GlomAction_Menu_Developer_Layout' />"
     "        <menuitem action='GlomAction_Menu_Developer_Users' />"
     "        <menuitem action='GlomAction_Menu_Developer_Reports' />"
+    "        <menuitem action='GlomAction_Menu_Developer_Script_Library' />"
     "        <separator />"
     "        <menuitem action='GlomAction_Menu_Developer_Translations' />"
     "        <menuitem action='GlomAction_Menu_Developer_ChangeLanguage' />"
@@ -399,6 +406,224 @@ void App_Glom::on_menu_userlevel_operator()
     m_pFrame->on_menu_userlevel_Operator(m_action_menu_userlevel_operator);
 }
 
+void App_Glom::on_menu_file_close() //override
+{
+  // Call the base class implementation:
+  Bakery::App_WithDoc_Gtk::on_menu_file_close();
+}
+
+static bool uri_is_writable(const Glib::RefPtr<const Gnome::Vfs::Uri>& uri)
+{
+  if(!uri)
+    return false;
+
+  Glib::RefPtr<const Gnome::Vfs::FileInfo> file_info = uri->get_file_info(Gnome::Vfs::FILE_INFO_GET_ACCESS_RIGHTS);
+  if(file_info)
+  {
+    const Gnome::Vfs::FilePermissions permissions = file_info->get_permissions();
+    return ((permissions & Gnome::Vfs::PERM_ACCESS_WRITABLE) == Gnome::Vfs::PERM_ACCESS_WRITABLE);
+  }
+  else
+    return true; //Not every URI protocol supports FILE_INFO_GET_ACCESS_RIGHTS, so assume that it's writable and complain later.
+}
+
+
+Glib::ustring App_Glom::get_file_uri_without_extension(const Glib::ustring& uri)
+{
+  Glib::RefPtr<Gnome::Vfs::Uri> vfs_uri = Gnome::Vfs::Uri::create(uri);
+  if(!vfs_uri)
+    return uri; //Actually an error.
+
+  const Glib::ustring filename_part = vfs_uri->extract_short_name();
+  
+  const Glib::ustring::size_type pos_dot = filename_part.rfind(".");
+  if(pos_dot == Glib::ustring::npos)
+    return uri; //There was no extension, so just return the existing URI.
+  else
+  {
+    const Glib::ustring filename_part_without_ext = filename_part.substr(0, pos_dot);
+    const Glib::ustring uri_parent = vfs_uri->extract_dirname();
+
+    Glib::RefPtr<Gnome::Vfs::Uri> vfs_uri_without_extension = Gnome::Vfs::Uri::create(uri_parent);
+    vfs_uri_without_extension->append_string(filename_part_without_ext);
+
+    return vfs_uri_without_extension->to_string();
+  }
+}
+
+Glib::ustring App_Glom::ui_file_select_save(const Glib::ustring& old_file_uri) //override
+{
+  //Reimplement this whole function, just so we can use our custom FileChooserDialog class:
+  App& app = *this;
+
+  Glom::FileChooserDialog fileChooser_Save(gettext("Save Document"), Gtk::FILE_CHOOSER_ACTION_SAVE);
+  fileChooser_Save.set_do_overwrite_confirmation(); //Ask the user if the file already exists.
+
+  Gtk::Window* pWindow = dynamic_cast<Gtk::Window*>(&app);
+  if(pWindow)
+    fileChooser_Save.set_transient_for(*pWindow);
+
+  fileChooser_Save.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+  fileChooser_Save.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
+
+  fileChooser_Save.set_default_response(Gtk::RESPONSE_OK);
+
+  //This is the reason that we override this method:
+  if(!m_ui_save_extra_title.empty())
+    fileChooser_Save.set_title(m_ui_save_extra_title);
+
+  fileChooser_Save.set_extra_message(m_ui_save_extra_message);
+    
+  //This is the reason that we override this method:
+  if(!m_ui_save_extra_title.empty())
+    fileChooser_Save.set_title(m_ui_save_extra_title);
+
+  fileChooser_Save.set_extra_message(m_ui_save_extra_message);
+
+  //Start with something suitable:
+  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  g_assert(document);
+  const Glib::ustring filename = document->get_name(); //Get the filename without the path and extension.
+
+  //Avoid ".". TODO: Find out why it happens:
+  if(filename == ".")
+    m_ui_save_extra_newdb_title = Glib::ustring();
+  else
+    m_ui_save_extra_newdb_title = Utils::title_from_string( filename ); //Start with something suitable.
+
+  fileChooser_Save.set_extra_newdb_details(m_ui_save_extra_newdb_title, m_ui_save_extra_newdb_selfhosted); 
+
+
+  //Make the save dialog show the existing filename, if any:
+  if(!old_file_uri.empty())
+  {
+    //Just start with the parent folder,
+    //instead of the whole name, to avoid overwriting:
+    Glib::RefPtr<Gnome::Vfs::Uri> vfs_uri = Gnome::Vfs::Uri::create(old_file_uri);
+    if(vfs_uri)
+    {
+      Glib::ustring uri_parent = vfs_uri->extract_dirname();
+      fileChooser_Save.set_uri(uri_parent);
+    }
+  }
+
+
+  //bool tried_once_already = false;
+
+  bool try_again = true;
+  while(try_again)
+  {
+    try_again = false;
+
+    //Work around bug #330680 "GtkFileChooserDialog is too small when shown a second time.":
+    //(Commented-out because the workaround doesn't work)
+    /*
+    if(tried_once_already)
+    {
+      fileChooser_Save.set_default_size(-1, 600); 
+    }
+    else
+      tried_once_already = true;
+    */
+
+    const int response_id = fileChooser_Save.run();
+    fileChooser_Save.hide();
+    if(response_id != Gtk::RESPONSE_CANCEL)
+    {
+      const Glib::ustring uri_chosen = fileChooser_Save.get_uri();
+
+      //Change the URI, to put the file (and its data folder) in a folder:
+      const Glib::ustring uri = get_file_uri_without_extension(uri_chosen);
+
+      //Check whether the file exists, and that we have rights to it:
+      Glib::RefPtr<Gnome::Vfs::Uri> vfs_uri = Gnome::Vfs::Uri::create(uri);
+      if(!vfs_uri)
+        return Glib::ustring(); //Failure.
+
+
+      //If the file exists (the FileChooser offers a "replace?" dialog, so this is not possible.):
+      if(App_WithDoc::file_exists(uri))
+      {
+        //Check whether we have rights to the file to change it:
+        //Really, GtkFileChooser should do this for us.
+        if(!uri_is_writable(vfs_uri))
+        {
+           //Warn the user:
+           ui_warning(gettext("Read-only File."), gettext("You may not overwrite the existing file, because you do not have sufficient access rights."));
+           try_again = true; //Try again.
+           continue;
+        }
+      }
+
+      //Check whether we have rights to the directory, to create a new file in it:
+      //Really, GtkFileChooser should do this for us.
+      Glib::RefPtr<const Gnome::Vfs::Uri> vfs_uri_parent = vfs_uri->get_parent();
+      if(vfs_uri_parent)
+      {
+        if(!uri_is_writable(vfs_uri_parent))
+        {
+          //Warn the user:
+           ui_warning(gettext("Read-only Directory."), gettext("You may not create a file in this directory, because you do not have sufficient access rights."));
+           try_again = true; //Try again.
+           continue;
+        }
+      }
+
+      if(!try_again)
+      {
+        //Get the extra details from the extended save dialog:
+        m_ui_save_extra_newdb_title = fileChooser_Save.get_extra_newdb_details(m_ui_save_extra_newdb_selfhosted); 
+        if(m_ui_save_extra_newdb_title.empty())
+        {
+          Frame_Glom::show_ok_dialog(_("Database Title missing"), _("You must specify a title for the new database."), *this, Gtk::MESSAGE_ERROR);
+
+          try_again = true;
+        }
+      }
+ 
+      if(!try_again && m_ui_save_extra_newdb_selfhosted)
+      {
+        //Create the directory, so that file creation can succeed later:
+        //0770 means "this user and his group can read and write this "executable" (can add child files) directory".
+        //The 0 prefix means that this is octal.
+        try
+        {
+          Gnome::Vfs::Handle::make_directory(uri, 0770 /* leading zero means octal */);
+        }
+        catch(const Gnome::Vfs::exception&  ex)
+        {
+          std::cerr << "Error during make_directory(): " << ex.what() << std::endl;
+        }
+
+        //Add the filename (Note that the caller with add the extension if necessary, so we don't do it here.)
+        Glib::RefPtr<Gnome::Vfs::Uri> uri_with_ext = Gnome::Vfs::Uri::create(uri_chosen);
+        const Glib::ustring filename_part = uri_with_ext->extract_short_name();
+
+        //Add the filename part to the newly-created directory:
+        Glib::RefPtr<Gnome::Vfs::Uri> uri_whole = vfs_uri->append_string(filename_part);
+        return uri_whole->to_string();
+      }
+    }
+    else
+      return Glib::ustring(); //The user cancelled.
+  }
+
+  return Glib::ustring();
+}
+
+void App_Glom::stop_self_hosting_of_document_database()
+{
+  Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
+  if(pDocument && pDocument->get_connection_is_self_hosted())
+  {
+    ConnectionPool* connection_pool = ConnectionPool::get_instance();
+    if(!connection_pool)
+      return;
+
+    connection_pool->stop_self_hosting();
+  }
+}
+
 Bakery::App* App_Glom::new_instance() //Override
 {
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_main");
@@ -434,7 +659,9 @@ bool App_Glom::on_document_load()
   //m_pFrame->load_from_document();
   Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
   if(!pDocument)
+  {
     return false;
+  }
   else
   {
     //Connect signals:
@@ -456,26 +683,28 @@ bool App_Glom::on_document_load()
       const bool is_example = pDocument->get_is_example_file();
       if(pDocument->get_is_example_file())
       {
-          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Creating from example file.")), true,  Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE);
-          dialog.set_secondary_text(_("To use this example file you must save an editable copy of the file. A new database will also be created on the server.")); //TODO: Better explanation.
-          dialog.set_transient_for(*this);
+        pDocument->set_file_uri(Glib::ustring()); //Prevent it from defaulting to the read-only examples directory when offering saveas.
 
-          dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-          dialog.add_button(Gtk::Stock::SAVE_AS, Gtk::RESPONSE_OK);
+        //m_ui_save_extra_* are used by offer_saveas() if it's not empty:
+        m_ui_save_extra_title = _("Creating From Example File");
+        m_ui_save_extra_message = _("To use this example file you must save an editable copy of the file. A new database will also be created on the server.");
+        m_ui_save_extra_newdb_title = "TODO";
+        m_ui_save_extra_newdb_selfhosted = true;
+        offer_saveas();
+        m_ui_save_extra_message.clear();
+        m_ui_save_extra_title.clear();
 
-          const int response = dialog.run();
-          dialog.hide();
-          if(response == Gtk::RESPONSE_CANCEL)
-            return false;
-          else
-          {
-            pDocument->set_file_uri(Glib::ustring()); //Prevent it from defaulting to the read-only examples directory when offering saveas.
-            offer_saveas();
-            if(get_operation_cancelled())
-              return false;
-            else
-              pDocument->set_is_example_file(false);
-          }
+        //Get the results from the extended save dialog:
+        pDocument->set_database_title(m_ui_save_extra_newdb_title);
+        pDocument->set_connection_is_self_hosted(m_ui_save_extra_newdb_selfhosted);
+
+        m_ui_save_extra_newdb_title.clear();
+        m_ui_save_extra_newdb_selfhosted = false;
+
+        if(get_operation_cancelled())
+          return false;
+        else
+          pDocument->set_is_example_file(false);
       }
 
       //Warn about read-only files, because users will otherwise wonder why they can't use Developer mode:
@@ -503,6 +732,21 @@ bool App_Glom::on_document_load()
       {
         //Set the connection details in the ConnectionPool singleton.
         //The ConnectionPool will now use these every time it tries to connect.
+        if(pDocument->get_connection_is_self_hosted())
+        {
+          // TODO: sleep, to give postgres time to start?
+          connection_pool->set_self_hosted(pDocument->get_connection_self_hosted_directory_uri());
+
+          if(!is_example) /* It will be started later, after we have asked for the initial db name/title and created the files.*/
+          {
+            connection_pool->start_self_hosting(); //Stopped in on_menu_file_close().
+          }
+        }
+        else
+        {
+          connection_pool->set_self_hosted(std::string());
+        }
+
         connection_pool->set_host(pDocument->get_connection_server());
         connection_pool->set_user(pDocument->get_connection_user());
         connection_pool->set_database(pDocument->get_connection_database());
@@ -514,12 +758,21 @@ bool App_Glom::on_document_load()
         {
           bool test = false;
           if(is_example)
+          {
+            //The user has already had the chance to specify a new filename and database name.
             test = m_pFrame->connection_request_password_and_choose_new_database_name();
+          }
           else
             test = m_pFrame->connection_request_password_and_attempt();
 
           if(!test) //It usually throws an exception instead of returning false.
+          {
+            //Stop self-hosting, if we are doing that:
+            std::cout << "debug: calling stop_self_hosting_of_document_database()" << std::endl;
+            stop_self_hosting_of_document_database();
+
             return false; //Failed. Close the document.
+          }
 
           if(is_example)
           {
@@ -672,7 +925,6 @@ bool App_Glom::offer_new_or_existing()
     if(!file_uri.empty())
       open_document(file_uri);
 
-
     //Check that a document was opened:
     Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
     if(document->get_file_uri().empty())
@@ -684,101 +936,86 @@ bool App_Glom::offer_new_or_existing()
   else if(response_id == 2) //New
   {
     //Each document must have a location, so ask the user for one.
+    //This will use an extended save dialog that also asks for the database title and some hosting details:
+    Glib::ustring db_title;
+
     offer_saveas();
 
     //Check that the document was given a location:
     Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
     if(!document->get_file_uri().empty())
     {
+      //Get details from the extended save dialog:
+      const Glib::ustring db_title = m_ui_save_extra_newdb_title;
+      const bool self_hosted = m_ui_save_extra_newdb_selfhosted;
+      m_ui_save_extra_newdb_title.clear();
+      m_ui_save_extra_newdb_selfhosted = false;
+
       //Make sure that the user can do something with his new document:
       document->set_userlevel(AppState::USERLEVEL_DEVELOPER);
 
       //Each new document must have an associated new database,
       //so ask the user for the name of one to create:
-      Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_new_database");
-      if(refXml)
+      
+
+      bool keep_asking = true;
+      while(keep_asking)
       {
-        Dialog_NewDatabase* dialog = 0;
-        refXml->get_widget_derived("dialog_new_database", dialog);
-        if(dialog)
+        //Create a database name based on the title.
+        //The user will (almost) never see this anyway but it's nicer than using a random number:
+        Glib::ustring db_name = Utils::create_name_from_title(db_title);
+
+        //Prefix glom_ to the database name, so it's more obvious
+        //for the system administrator.
+        //This database name should never be user-visible again, either prefixed or not prefixed.
+        db_name = "glom_" + db_name;
+
+        if(!db_name.empty()) //The dialog and prefix prevent this anyway.
         {
-          std::auto_ptr<Dialog_NewDatabase> dialog_owner(dialog); //This will delete the dialog even when we return in the middle of this function.
-          dialog->set_transient_for(*this);
+          //Connect to the server and choose a variation of this db_name that does not exist yet:
+          document->set_connection_database(db_name);
+          document->set_connection_is_self_hosted(self_hosted);
+               
+          const bool connected = m_pFrame->connection_request_password_and_choose_new_database_name();
+          if(!connected)
+            return false;
 
-          //Set suitable defaults:
-          const Glib::ustring filename = document->get_name(); //Get the filename without the path and extension.
-          dialog->set_input( Utils::title_from_string( filename ) ); //Start with something suitable.
-
-          bool keep_asking = true;
-          while(keep_asking)
+          const bool db_created = m_pFrame->create_database(document->get_connection_database(), db_title, false /* do not request password */);
+          if(db_created)
           {
-            int response = Glom::Utils::dialog_run_with_help(dialog, "dialog_new_database");
+            keep_asking = false;
 
-            if(response == Gtk::RESPONSE_OK)
+            //document->set_connection_database(db_name); //Select the database that was just created.
+
+            /*
+            ConnectionPool* connection_pool = ConnectionPool::get_instance();
+            if(connection_pool)
             {
-              Glib::ustring db_title;
-              dialog->get_input(db_title);
-
-              //Create a database name based on the title.
-              //The user will (almost) never see this anyway but it's nicer than using a random number:
-              Glib::ustring db_name = Utils::create_name_from_title(db_title);
-
-              //Prefix glom_ to the database name, so it's more obvious
-              //for the system administrator.
-              //This database name should never be user-visible again, either prefixed or not prefixed.
-              db_name = "glom_" + db_name;
-
-              if(!db_name.empty()) //The dialog prevents this anyway.
-              {
-                //Connect to the server and choose a variation of this db_name that does not exist yet:
-                document->set_connection_database(db_name);
-                const bool connected = m_pFrame->connection_request_password_and_choose_new_database_name();
-                if(!connected)
-                  return false;
-
-                const bool db_created = m_pFrame->create_database(document->get_connection_database(), db_title, false /* do not request password */);
-                if(db_created)
-                {
-                  keep_asking = false;
-
-                  //document->set_connection_database(db_name); //Select the database that was just created.
-
-                  /*
-                  ConnectionPool* connection_pool = ConnectionPool::get_instance();
-                  if(connection_pool)
-                  {
-                    connection_pool->set_database(db_name); //The rest has been set while creating the database.
-                  }
-                  */
-
-                  const Glib::ustring database_name_used = document->get_connection_database();
-                  ConnectionPool::get_instance()->set_database(database_name_used);
-                  document->set_database_title(db_title);
-                  m_pFrame->set_databases_selected(database_name_used);
-                }
-                else
-                {
-                  //Ask again:
-                  return offer_new_or_existing();
-                }
-              }
-              else
-              {
-                g_warning(" App_Glom::offer_new_or_existing(): db_name is empty.");
-                //And ask again, by going back to the start of the while() loop.
-              }
-            } /* if(response) */
-            else
-            {
-              return false; //The user cancelled.
+              connection_pool->set_database(db_name); //The rest has been set while creating the database.
             }
-          } /* while() */
+            */
 
-          return true; //File successfully created.
+            const Glib::ustring database_name_used = document->get_connection_database();
+            ConnectionPool::get_instance()->set_database(database_name_used);
+            document->set_database_title(db_title);
+            m_pFrame->set_databases_selected(database_name_used);
+          }
+          else
+          {
+            //Ask again:
+            return offer_new_or_existing();
+          }
+        }
+        else
+        {
+           g_warning(" App_Glom::offer_new_or_existing(): db_name is empty.");
+           //And ask again, by going back to the start of the while() loop.
         }
-       }
 
-      return false; //Creation of new document failed.
+      } /* while() */
+
+      return true; //File successfully created.
+
     }
     else
     {
@@ -797,7 +1034,6 @@ bool App_Glom::offer_new_or_existing()
 
   return true;
 }
-
 void App_Glom::set_mode_data()
 {
   m_action_mode_data->activate();
@@ -878,14 +1114,15 @@ bool App_Glom::recreate_database(bool& user_cancelled)
   //Show the user that something is happening, because the INSERTS might take time.
   //TOOD: This doesn't actually show up until near the end, even with Gtk::Main::instance()->iteration().
   std::auto_ptr<Dialog_ProgressCreating> dialog_progress;
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_progress_creating_from_example");
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_progress");
   if(refXml)
   {
     Dialog_ProgressCreating* dialog_progress_temp = 0;
-    refXml->get_widget_derived("window_progress_creating_from_example", dialog_progress_temp);
+    refXml->get_widget_derived("window_progress", dialog_progress_temp);
     if(dialog_progress_temp)
     {
-      dialog_progress.reset(dialog_progress_temp); //The dialog will be deleted (and hidden) when this function returns.
+      dialog_progress_temp->set_message(_("Creating Glom Database"), _("Creating Glom database from example file."));
+      dialog_progress.reset(dialog_progress_temp); //Put the dialog in an auto_ptr so that it will be deleted (and hidden) when the current function returns.
 
       dialog_progress->set_transient_for(*this);
       dialog_progress->show();
