@@ -87,6 +87,8 @@ protected:
 
   virtual void on_menu_file_close(); //override.
 
+  virtual Glib::ustring ui_file_select_save(const Glib::ustring& old_file_uri); //override.
+
   virtual void on_userlevel_changed(AppState::userlevels userlevel);
 
   virtual Bakery::App* new_instance(); //Override
@@ -94,6 +96,8 @@ protected:
   bool recreate_database(bool& user_cancelled); //return indicates success.
   void stop_self_hosting_of_document_database();
 
+  static Glib::ustring get_file_uri_without_extension(const Glib::ustring& uri);
+
   typedef Bakery::App_WithDoc_Gtk type_base;
 
   //Widgets:
@@ -114,6 +118,14 @@ protected:
   type_listActions m_listNavTableActions, m_listNavReportActions;
   Gtk::UIManager::ui_merge_id m_menu_tables_ui_merge_id, m_menu_reports_ui_merge_id;
 
+  //Set these before calling offer_saveas() (which uses ui_file_select_save()), and clear it afterwards.
+  Glib::ustring m_ui_save_extra_title;
+  Glib::ustring m_ui_save_extra_message;
+
+  Glib::ustring m_ui_save_extra_newdb_title;
+  bool m_ui_save_extra_newdb_selfhosted;
+
+
   bool m_show_sql_debug;
 };
 
