@@ -37,11 +37,11 @@ Box_Data_List::Box_Data_List()
 {
   m_layout_name = "list";
 
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout_list"); //TODO: Use a generic layout dialog?
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout"); //TODO: Use a generic layout dialog?
   if(refXml)
   {
     Dialog_Layout_List* dialog = 0;
-    refXml->get_widget_derived("window_data_layout_list", dialog);
+    refXml->get_widget_derived("window_data_layout", dialog);
     if(dialog)
     {
       m_pDialogLayout = dialog;
