@@ -24,6 +24,9 @@
 #include <gtkmm.h>
 #include <libglademm.h>
 
+namespace Glom
+{
+
 class RelationshipsCanvasTableWidget : public Gtk::Alignment
 {
 public:
@@ -32,4 +35,6 @@ public:
   virtual ~RelationshipsCanvasTableWidget();
 };
 
+} //namespace Glom
+
 #endif //GLOM_RELATIONSHIPS_OVERVIEW_TABLE_WIDGET_H
